ARCHITECTURE test OF ahbGpio_tester IS
                                                              -- reset and clock
  constant clockFrequency: real := 100.0E6;
  constant clockPeriod: time := (1.0/clockFrequency) * 1 sec;
  signal clock_int: std_uLogic := '1';
  signal reset_int: std_uLogic;
                                                          -- register definition
  constant dataRegisterAddress: natural := 0;
  constant outputEnableRegisterAddress: natural := 1;
                                                              -- AMBA bus access
  signal registerAddress: natural;
  signal registerData: integer;
  signal registerWrite: std_uLogic;
  signal registerRead: std_uLogic;
                                                                  -- GPIO access
  signal ioData: integer;
  signal ioMask: integer;

BEGIN
  ------------------------------------------------------------------------------
                                                              -- reset and clock
  reset_int <= '1', '0' after 2*clockPeriod;
  hReset_n <= not(reset_int);

  clock_int <= not clock_int after clockPeriod/2;
  hClk <= transport clock_int after clockPeriod*9.0/10.0;

  ------------------------------------------------------------------------------
                                                                -- test sequence
  testSequence: process
  begin
    registerAddress <= 0;
    registerData <= 0;
    registerWrite <= '0';
    registerRead <= '0';
    ioData <= 0;
    ioMask <= 0;
    wait for 100 ns;
                                                                -- write en mask
    ioData <= 16#AA#;
    ioMask <= 16#0F#; wait for 0 ns;
    registerAddress <= outputEnableRegisterAddress;
    registerData <= ioMask;
    registerWrite <= '1', '0' after clockPeriod;
    wait for 4*clockPeriod;
                                                        -- write output data 55h
    registerAddress <= dataRegisterAddress;
    registerData <= 16#55#;
    registerWrite <= '1', '0' after clockPeriod;
    wait for 4*clockPeriod;
                                                                    -- read data
    registerAddress <= dataRegisterAddress;
    registerRead <= '1', '0' after clockPeriod;
    wait for 4*clockPeriod;
                                                            -- end of simulation
    wait;
  end process testSequence;

  ------------------------------------------------------------------------------
                                                              -- AMBA bus access
  busAccess: process
    variable writeAccess: boolean;
  begin
    hAddr <= (others => '-');
    hWData <= (others => '-');
    hTrans <= transIdle;
    hSel <= '0';
    hWrite <= '0';
    wait on registerWrite, registerRead;
    writeAccess := false;
    if rising_edge(registerWrite) then
      writeAccess := true;
    end if;
                                                -- phase 1: address and controls
    wait until rising_edge(clock_int);
    hAddr <= to_unsigned(registerAddress, hAddr'length);
    hTrans <= transNonSeq;
    hSel <= '1';
    if writeAccess then
      hWrite <= '1';
    end if;
                                                                -- phase 2: data
    wait until rising_edge(clock_int);
    hAddr <= (others => '-');
    hTrans <= transIdle;
    hSel <= '0';
    hWrite <= '0';
    if writeAccess then
      hWData <= std_uLogic_vector(to_signed(registerData, hWData'length));
    end if;
    wait until rising_edge(clock_int);
  end process;


  ------------------------------------------------------------------------------
                                                                  -- GPIO access
  linesAccess: process(ioData, ioMask)
    variable ioDataVector: unsigned(io'range);
    variable ioMaskVector: unsigned(io'range);
  begin
    ioDataVector := to_unsigned(ioData, ioDataVector'length);
    ioMaskVector := to_unsigned(ioMask, ioMaskVector'length);
    for index in io'range loop
      if ioMaskVector(index) = '1' then
        io(index) <= 'Z';
      else
        io(index) <= ioDataVector(index);
      end if;
    end loop;
  end process;

END ARCHITECTURE test;
